"""Author: MrNaif2018
Email: chuff184@gmail.com"""
__author__ = "MrNaif2018"
__email__ = "chuff184@gmail.com"
try:
    import requests
except ImportError:
    raise ImportError("You must install requests library first!")
try:
    from simplejson import loads as json_loads
except (ImportError, ValueError):
    from json import loads as json_loads
import warnings
from typing import Any, Optional, Union, Callable


class RPCProxy:
    def __init__(
        self: "RPCProxy",
        url: str,
        username: Optional[str] = None,
        password: Optional[str] = None,
        xpub: Optional[str] = None,
        session: Optional[requests.Session] = None,
        verify: Optional[bool] = True,
    ):
        self.url = url
        self.username = username
        self.password = password
        self.xpub = xpub
        self.verify = verify
        self.session = session or requests.Session()

    def _send_request(self: "RPCProxy", method: str, *args: Any, **kwargs: Any) -> Any:
        if not self.username or not self.password:
            auth = None
        else:
            auth = (self.username, self.password)
        arg: Union[dict, tuple]
        if args:
            arg = args
        elif kwargs:
            arg = kwargs
        else:
            arg = ()
        dict_to_send = {"id": 0, "method": method, "params": arg}
        if self.xpub:
            dict_to_send["xpub"] = self.xpub
        response = self.session.post(
            self.url,
            headers={"content-type": "application/json"},
            json=dict_to_send,
            auth=auth,
            verify=self.verify,
        )
        response.raise_for_status()
        json = response.json()
        if json["error"]:
            raise ValueError("Error from server: {}".format(json["error"]))
        if json["id"] != 0:
            warnings.warn("ID mismatch!")
        result = json.get("result", {})
        if isinstance(result, str):
            try:
                result = json_loads(result)
            except Exception:
                pass
        return result

    def __getattr__(
        self: "RPCProxy", method: str, *args: Any, **kwargs: Any
    ) -> Callable:
        def wrapper(*args: Any, **kwargs: Any) -> Any:
            return self._send_request(method, *args, **kwargs)

        return wrapper



import importlib
from typing import Dict, Iterable, Union
from types import ModuleType

SYSTEM_PACKAGES = ["electrum"]


class Coin:
    """Coins should reimplement some methods,
    and initialize coin-specific info.
    Required information is:
    coin_name str
    friendly_name str
    providers list
    For more info see the docs.
    """

    coin_name = "Base"
    friendly_name = "Base"
    providers: Union[Iterable[str], Dict[str, ModuleType]] = []

    def __init__(self: "Coin") -> None:
        # Initialize the providers.
        self.providers_new: Dict[str, ModuleType] = {}
        for i in self.providers:
            if i not in SYSTEM_PACKAGES:
                self.providers_new[i] = importlib.import_module(
                    ".providers." + i, "bitcart"
                )
            else:
                self.providers_new[i] = importlib.import_module(i)
        self.providers = self.providers_new
        del self.providers_new

    def help(self) -> list:
        """Get help

        Returns a list of all available RPC methods

        Raises:
            NotImplementedError: Implement in your subclass

        Returns:
            list: RPC methods list
        """
        raise NotImplementedError()

    def get_tx(self, tx: str) -> dict:
        """Get transaction information

        Given tx hash of transaction, return full information as dictionary

        Example:

        >>> c.get_tx("54604b116b28124e31d2d20bbd4561e6f8398dca4b892080bffc8c87c27762ba")
        {'partial': False, 'version': 2, 'segwit_ser': True, 'inputs': [{'prevout_hash': 'xxxx',...

        Args:
            tx (str): tx_hash

        Raises:
            NotImplementedError: Implement in your subclass

        Returns:
            dict: transaction info
        """
        raise NotImplementedError()

    def get_address(self, address: str) -> list:
        """Get address history

        This method should return list of transaction informations for specified address

        Example:

        >>> c.get_address("31smpLFzLnza6k8tJbVpxXiatGjiEQDmzc")
        [{'tx_hash': '7854bdf4c4e27276ecc1fb8d666d6799a248f5e81bdd58b16432d1ddd1d4c332', 'height': 581878, 'tx': {'partial': False,...

        Args:
            address (str): address to get transactions for

        Raises:
            NotImplementedError: Override this method in subclass

        Returns:
            list: List of transactions
        """
        raise NotImplementedError()

    def balance(self) -> dict:
        """Get balance of wallet

        Example:

        >>> self.balance()
        {"confirmed": "0.00005", "unconfirmed": 0, "unmatured": 0}

        Raises:
            NotImplementedError: Implement in your subclass

        Returns:
            dict: It should return dict of balance statuses
        """
        raise NotImplementedError()



from setuptools import setup, find_packages

setup(
    name="bitcart",
    packages=find_packages(),
    version="0.4.0",
    license="MIT",
    description="Bitcart coins support library",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    author="MrNaif2018",
    author_email="chuff184@gmail.com",
    url="https://github.com/MrNaif2018/bitcart-sdk/",
    download_url="https://github.com/MrNaif2018/bitcart-sdk/archive/0.1.tar.gz",
    keywords=["electrum", " daemon", "bitcart"],
    install_requires=["requests", "simplejson"],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
)
